'**********************************************************************************************
'   fzxBareBones
'**********************************************************************************************
'$INCLUDE:'include/raylib.bi'
'$include:'fzxNGN_BASE_v2\fzxNGN_ini.bas'

' This is the maximum amount of elements (quads) per batch
' NOTE: This value is defined in [rlgl] module and can be changed there
CONST MAX_BATCH_ELEMENTS = 8192

InitWindow 1024, 768, "raylib fzxNGN example"
DIM SHARED AS LONG iterations: iterations = 100
DIM SHARED AS DOUBLE dt: dt = 1 / 60

'**********************************************************************************************
' Build the playfield
'**********************************************************************************************
SetTargetFPS 60 ' Set our game to run at 60 frames-per-second
buildScene

'**********************************************************************************************
' This is the main loop
'**********************************************************************************************

DO
  animatescene
  fzxImpulseStep dt, iterations
  DrawText "Click the mouse on the playfield to spawn an object", 120, 10, 20, BLACK
  DrawText "Throw objects by moving mouse and releasing the left mouse button", 120, 40, 20, BLACK
  renderBodies
LOOP UNTIL WindowShouldClose

SYSTEM


' This provides access to all of the fzxNGN functionality
'$include:'fzxNGN_BASE_v2\fzxNGN_BASE.bas'
'$INCLUDE:'include/raylib.bas'
'**********************************************************************************************
'    This is where you interact with the world.
'**********************************************************************************************

SUB animatescene
  DIM AS LONG temp
  DIM AS Vector2 position, vel

  ' Create a object on mouse click
  'IF __fzxInputDevice.mouse.b1.NegEdge THEN
  IF (IsMouseButtonReleased(MOUSE_BUTTON_LEFT)) THEN
    ' Drop a ball or a poly, flip a coin
    IF RND > .5 THEN
      temp = fzxCreateCircleBodyEx("b" + _TRIM$(STR$(RND * 1000000000)), 10 * RND + 10)
    ELSE
      temp = fzxCreatePolyBodyEx("b" + _TRIM$(STR$(RND * 1000000000)), 10 * RND + 10, 10 * RND + 10, 3 + INT(RND * 5))
    END IF
    ' Set the bodies parameters
    ' Put the body where the mouse is on the screen
    GetMousePosition position
    GetMouseDelta vel
    fzxSetBody cFZX_PARAMETER_POSITION, temp, position.x, position.y
    ' Give it the mouse's velocity, so you can throw it
    fzxSetBody cFZX_PARAMETER_VELOCITY, temp, vel.x * 10, vel.y * 10
    ' Change its orientation or angle
    fzxSetBody cFZX_PARAMETER_ORIENT, temp, _D2R(RND * 360), 0
    ' Set the bouncyness
    fzxSetBody cFZX_PARAMETER_RESTITUTION, temp, .5, 0 ' Bounce
    ' Set the friction values of the body
    fzxSetBody cFZX_PARAMETER_STATICFRICTION, temp, .9, 0
    fzxSetBody cFZX_PARAMETER_DYNAMICFRICTION, temp, .25, 0
    ' Bodies wont live forever
    fzxSetBody cFZX_PARAMETER_LIFETIME, temp, RND * 20 + 10, 0
    fzxSetBody cFZX_PARAMETER_COLOR, temp, _RGB32(RND * 200, RND * 200, RND * 200), 0
  END IF

END SUB

'********************************************************
'   Build you world here
'********************************************************

SUB buildScene
  DIM AS LONG temp

  '********************************************************
  '   Setup World
  '********************************************************

  fzxVector2DSet __fzxWorld.minusLimit, -200000, -200000
  fzxVector2DSet __fzxWorld.plusLimit, 200000, 200000
  fzxVector2DSet __fzxWorld.spawn, 0, 0
  fzxVector2DSet __fzxWorld.gravity, 0.0, 100.0

  ' Some math used on the impulse side
  ' Todo: move this elsewhere
  DIM o AS tFZX_VECTOR2d
  fzxVector2DMultiplyScalarND o, __fzxWorld.gravity, dt
  __fzxWorld.resting = fzxVector2DLengthSq(o) + cFZX_EPSILON

  '********************************************************
  '   Build Level
  '********************************************************

  temp = fzxCreateBoxBodyEx("floor", 800, 10)
  fzxSetBody cFZX_PARAMETER_POSITION, temp, __fzxWorld.spawn.x + 400, __fzxWorld.spawn.y + 700
  fzxSetBody cFZX_PARAMETER_STATIC, temp, 0, 0
  fzxSetBody cFZX_PARAMETER_COLOR, temp, _RGB32(39), 0

END SUB

SUB renderBodies
  DIM i AS LONG

  DIM AS LONG ub: ub = UBOUND(__fzxBody)
  BeginDrawing
  ClearBackground RAYWHITE
  'Draw all of the bodies that are visible
  i = 0: DO WHILE i < ub
    IF __fzxBody(i).enable AND __fzxBody(i).objectHash <> 0 THEN
      IF __fzxBody(i).shape.ty = cFZX_SHAPE_CIRCLE THEN
        renderWireFrameCircleNC i
      ELSE IF __fzxBody(i).shape.ty = cFZX_SHAPE_POLYGON THEN
          renderWireFramePolyNC i
        END IF
      END IF
    END IF
    i = i + 1
  LOOP
  DrawFPS 10, 10
  EndDrawing
END SUB

SUB renderWireFrameCircleNC (index AS LONG) ' No camera
  DIM AS tFZX_VECTOR2d o1, o2
  o1 = __fzxBody(index).fzx.position
  DrawCircleLines o1.x, o1.y, __fzxBody(index).shape.radius, __fzxBody(index).c
  o2.x = o1.x + __fzxBody(index).shape.radius * COS(__fzxBody(index).fzx.orient)
  o2.y = o1.y + __fzxBody(index).shape.radius * SIN(__fzxBody(index).fzx.orient)
  DrawLine o1.x, o1.y, o2.x, o2.y, __fzxBody(index).c
END SUB

SUB renderWireFramePolyNC (index AS LONG) 'No camera
  DIM AS LONG polyCount, i
  polyCount = fzxGetBodyD(CFZX_PARAMETER_POLYCOUNT, index, 0)
  DIM AS tFZX_VECTOR2d vert1, vert2
  i = 0: DO WHILE i <= polyCount
    ' Pull vertices from the bodyies buffer
    fzxGetBodyVert index, i, vert1
    fzxGetBodyVert index, fzxArrayNextIndex(i, polyCount), vert2
    ' Rotate vertices
    fzxMatrix2x2MultiplyVector __fzxBody(index).shape.u, vert1, vert1
    fzxMatrix2x2MultiplyVector __fzxBody(index).shape.u, vert2, vert2
    ' Offset Position
    fzxVector2DAddVector vert1, __fzxBody(index).fzx.position
    fzxVector2DAddVector vert2, __fzxBody(index).fzx.position

    DrawLine vert1.x, vert1.y, vert2.x, vert2.y, __fzxBody(index).c
  i = i + 1: LOOP
END SUB


